/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 2; tab-width: 2 -*- */
/*
 * main_window.cc
 * Copyright (C) Mike Crash 2008 <mike@mikecrash.com>
 *
 * main.cc is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * main.cc is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <iostream>
#include <sigc++/class_slot.h>
#include <glibmm.h>
#include <config.h>
#include <stdio.h>  
#include <time.h>  
#include <sys/time.h>
#include "main_window.h"

#define LOGO_FILE "chronometer.png"

// Constructor
MainWindow::MainWindow()
{
	CreateWidgets();
	sigc::slot<bool> my_slot = sigc::bind(sigc::mem_fun(*this, &MainWindow::on_timeout), 1);
	sigc::connection conn = Glib::signal_timeout().connect(my_slot, 137);
	mode = 0;
	split = 0;
}

// Destructor
MainWindow::~MainWindow()
{

}

void MainWindow::CreateWidgets(void)
{
  // The window
  set_title("Chronometer");
  set_default_size( 300 , 400 );
  set_icon_from_file(PIXMAPS_DIR "/" LOGO_FILE);

	//main box
  Gtk::VBox *main_vbox = manage (new Gtk::VBox(false, 4));
  add(*main_vbox);
    //chrono
    label_chrono = manage ( new Gtk::Label("00:00.00", false));
    main_vbox->pack_start(*label_chrono, false, false, 0);
		Pango::FontDescription chrono_font("Monospace bold 48");
		label_chrono->modify_font(chrono_font);
    //time
    Gtk::HBox *time_hbox = manage (new Gtk::HBox(false,2));
    main_vbox->pack_start(*time_hbox, false, false, 0);
    label_split = manage ( new Gtk::Label("0:00:00", false));
    time_hbox->pack_start(*label_split, true, false, 0);
    label_time = manage ( new Gtk::Label("0:00", false));
    time_hbox->pack_start(*label_time, true, false, 0);
		Pango::FontDescription time_font("Monospace bold 20");
		label_split->modify_font(time_font);
		label_time->modify_font(time_font);
    //lap
    Gtk::HBox *lap_hbox = manage (new Gtk::HBox(true,2));
    main_vbox->pack_start(*lap_hbox, true, true, 0);
		//Gtk::VBox *lap_vbox1 = manage (new Gtk::VBox(false,2));
    //lap_hbox->pack_start(*lap_vbox1, true, true, 0);
			//laps
	    Gtk::Frame *frame_lap = manage ( new Gtk::Frame("Lap times"));
	    lap_hbox->pack_start(*frame_lap, true, true, 0);
			Gtk::ScrolledWindow *scroll = manage ( new Gtk::ScrolledWindow());
			frame_lap->add(*scroll);
			textview_laps = manage( new Gtk::TextView());
			scroll->add(*textview_laps);
			scroll->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
			textview_laps->set_editable(false);
			textbuffer_laps = textview_laps->get_buffer();
			Gtk::VBox *lap_vbox2 = manage (new Gtk::VBox(true,2));
      lap_hbox->pack_start(*lap_vbox2, true, true, 0);
      //clear button
      button_clear = manage (new Gtk::Button());
      button_clear->set_label("Clear");
      lap_vbox2->pack_start(*button_clear, true, true, 0);
			button_clear->set_sensitive(true);
      button_clear->signal_pressed().connect ( sigc::mem_fun(*this, &MainWindow::on_button_clear_clicked) );
      //lap button
      button_lap = manage (new Gtk::Button());
      button_lap->set_label("Lap");
      lap_vbox2->pack_start(*button_lap, true, true, 0);
			button_lap->set_sensitive(true);
      button_lap->signal_pressed().connect ( sigc::mem_fun(*this, &MainWindow::on_button_lap_clicked) );
		//buttons
    Gtk::HBox *but_hbox = manage (new Gtk::HBox(true,2));
    main_vbox->pack_start(*but_hbox, true, true, 0);
      //split button
      button_split = manage (new Gtk::Button());
      button_split->set_label("Split");
      but_hbox->pack_start(*button_split, true, true, 0);
			button_split->set_sensitive(true);
      button_split->signal_pressed().connect ( sigc::mem_fun(*this, &MainWindow::on_button_split_clicked) );
      //start button
      button_start = manage (new Gtk::Button());
      button_start->set_label("Start");
      but_hbox->pack_start(*button_start, true, true, 0);
			button_start->set_sensitive(true);
      button_start->signal_pressed().connect ( sigc::mem_fun(*this, &MainWindow::on_button_start_clicked) );

    show_all();
}

void MainWindow::CreateTime(char *s, struct timeval*start, struct timeval*stop)
{
	long mtime, hours, minutes, seconds, miliseconds, useconds;
	seconds = stop->tv_sec - start->tv_sec;
	useconds = stop->tv_usec - start->tv_usec;
	mtime = seconds * 1000 + useconds/1000;
	hours = mtime / 3600000;
	mtime -= hours * 3600000;
	minutes = mtime / 60000;
	mtime -= minutes * 60000;
	seconds = mtime / 1000;
	miliseconds = mtime - seconds * 1000;
	miliseconds = miliseconds / 10;
	if (hours)
    sprintf(s, "%02d:%02d:%02d", hours, minutes, seconds);
	else
    sprintf(s, "%02d:%02d.%02d", minutes, seconds, miliseconds);	
}

// Signal
void MainWindow::on_button_lap_clicked()
{
  if (mode==1)
	{
	  char timestamp[30];
		Gtk::TextBuffer::iterator textend;
	  struct timeval time_cur;
		gettimeofday(&time_cur, NULL);
		CreateTime(timestamp, &time_start, &time_cur);
		int cnt = textbuffer_laps->get_line_count();
		textend = textbuffer_laps->get_iter_at_line_index(cnt,0);
		textend = textbuffer_laps->insert(textend, timestamp);
		textend = textbuffer_laps->insert(textend, " / ");
		CreateTime(timestamp, &time_lap, &time_cur);
		textend = textbuffer_laps->insert(textend, timestamp);
		textend = textbuffer_laps->insert(textend, "\n");
		time_lap = time_cur;
	}
}

void MainWindow::on_button_start_clicked()
{
  struct timeval time_cur;
  char timestamp[30];

	switch (mode)
	{
		case 0:
			gettimeofday(&time_start, NULL);
			time_lap = time_start;
			mode = 1;
			button_start->set_label("Stop");
			break;
		case 1:
			gettimeofday(&time_stop, NULL);
			button_start->set_label("Start");
			mode = 2;
			CreateTime(timestamp, &time_start, &time_stop);
			label_chrono->set_label(timestamp);
			break;
		case 2:
			gettimeofday(&time_cur, NULL);
			time_start.tv_sec = time_cur.tv_sec-(time_stop.tv_sec-time_start.tv_sec);
			time_start.tv_usec = time_cur.tv_usec-(time_stop.tv_usec-time_start.tv_usec);
			mode = 1;
			if (split)
			{
				time_split.tv_sec = time_cur.tv_sec-(time_stop.tv_sec-time_split.tv_sec);
				time_split.tv_usec = time_cur.tv_usec-(time_stop.tv_usec-time_split.tv_usec);
			}
			button_start->set_label("Stop");
			break;
	}
}

void MainWindow::on_button_clear_clicked()
{
	if (mode==0)
		textbuffer_laps->set_text("");
	mode = 0;
	split = 0;
	label_chrono->set_label("00:00.00");
	label_split->set_label("00:00.00");
	button_start->set_label("Start");
}

void MainWindow::on_button_split_clicked()
{
  if (mode==1)
	{
	  char timestamp[30];
		split = 1;
		gettimeofday(&time_split, NULL);
		CreateTime(timestamp, &time_start, &time_split);
		label_split->set_label(timestamp);
	}
}

bool MainWindow::on_timeout(int timer_number)
{
	struct timeval time_cur;
	long mtime, hours, minutes, seconds, miliseconds, useconds;
  char timestamp[30];

	gettimeofday(&time_cur, NULL);
	
  //update time
	if (last_sec != time_cur.tv_sec)
	{
		struct tm *mytm;
		last_sec = time_cur.tv_sec;
		mytm=localtime(&time_cur.tv_sec);
		strftime(timestamp,sizeof timestamp,"%X",mytm);
		label_time->set_label(timestamp);
	}
	
	//update chrono
	if (mode == 1)
	{
		CreateTime(timestamp, &time_start, &time_cur);
		label_chrono->set_label(timestamp);
	}
	
  // Keep going (do not disconnect yet):
  return true;
}